/*
 * Work under Copyright. Licensed under the EUPL.
 * See the project README.md and LICENSE.txt for more information.
 */

package net.dries007.tfc.objects.blocks;

import java.util.Random;
import javax.annotation.Nullable;
import javax.annotation.ParametersAreNonnullByDefault;

import net.minecraft.block.Block;
import net.minecraft.block.SoundType;
import net.minecraft.block.material.Material;
import net.minecraft.block.properties.PropertyInteger;
import net.minecraft.block.state.BlockFaceShape;
import net.minecraft.block.state.BlockStateContainer;
import net.minecraft.block.state.IBlockState;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.init.Items;
import net.minecraft.init.SoundEvents;
import net.minecraft.item.Item;
import net.minecraft.item.ItemStack;
import net.minecraft.util.EnumFacing;
import net.minecraft.util.EnumHand;
import net.minecraft.util.SoundCategory;
import net.minecraft.util.math.AxisAlignedBB;
import net.minecraft.util.math.BlockPos;
import net.minecraft.util.math.RayTraceResult;
import net.minecraft.world.IBlockAccess;
import net.minecraft.world.World;

import mcp.MethodsReturnNonnullByDefault;
import net.dries007.tfc.util.Helpers;

@ParametersAreNonnullByDefault
@MethodsReturnNonnullByDefault
public class BlockCharcoalPile extends Block
{
    public static final PropertyInteger LAYERS = PropertyInteger.func_177719_a("type", 1, 8);
    private static final AxisAlignedBB[] PILE_AABB = new AxisAlignedBB[] {
        new AxisAlignedBB(0.0D, 0.0D, 0.0D, 1.0D, 0.0D, 1.0D),
        new AxisAlignedBB(0.0D, 0.0D, 0.0D, 1.0D, 0.125D, 1.0D),
        new AxisAlignedBB(0.0D, 0.0D, 0.0D, 1.0D, 0.25D, 1.0D),
        new AxisAlignedBB(0.0D, 0.0D, 0.0D, 1.0D, 0.375D, 1.0D),
        new AxisAlignedBB(0.0D, 0.0D, 0.0D, 1.0D, 0.5D, 1.0D),
        new AxisAlignedBB(0.0D, 0.0D, 0.0D, 1.0D, 0.625D, 1.0D),
        new AxisAlignedBB(0.0D, 0.0D, 0.0D, 1.0D, 0.75D, 1.0D),
        new AxisAlignedBB(0.0D, 0.0D, 0.0D, 1.0D, 0.875D, 1.0D),
        new AxisAlignedBB(0.0D, 0.0D, 0.0D, 1.0D, 1.0D, 1.0D)};

    BlockCharcoalPile()
    {
        super(Material.field_151578_c);

        func_149672_a(SoundType.field_185849_b);
        setHarvestLevel("shovel", 0);
        func_149711_c(1.0F);
        this.func_180632_j(this.field_176227_L.func_177621_b().func_177226_a(LAYERS, 1));
    }

    @Override
    @SuppressWarnings("deprecation")
    public boolean func_185481_k(IBlockState state)
    {
        return state.func_177229_b(LAYERS) == 8;
    }

    @Override
    @SuppressWarnings("deprecation")
    public IBlockState func_176203_a(int meta)
    {
        return this.func_176223_P().func_177226_a(LAYERS, meta + 1);
    }

    @Override
    public int func_176201_c(IBlockState state)
    {
        return state.func_177229_b(LAYERS) - 1;
    }

    @Override
    @SuppressWarnings("deprecation")
    public boolean func_149686_d(IBlockState state)
    {
        return state.func_177229_b(LAYERS) == 8;
    }

    @Override
    @SuppressWarnings("deprecation")
    public AxisAlignedBB func_185496_a(IBlockState state, IBlockAccess source, BlockPos pos)
    {
        return PILE_AABB[state.func_177229_b(LAYERS)];
    }

    @Override
    @SuppressWarnings("deprecation")
    public BlockFaceShape func_193383_a(IBlockAccess worldIn, IBlockState state, BlockPos pos, EnumFacing face)
    {
        return face == EnumFacing.DOWN ? BlockFaceShape.SOLID : BlockFaceShape.UNDEFINED;
    }

    @Override
    @Nullable
    @SuppressWarnings("deprecation")
    public AxisAlignedBB func_180646_a(IBlockState blockState, IBlockAccess worldIn, BlockPos pos)
    {
        int i = blockState.func_177229_b(LAYERS) - 1;
        AxisAlignedBB axisalignedbb = blockState.func_185900_c(worldIn, pos);
        return new AxisAlignedBB(axisalignedbb.field_72340_a, axisalignedbb.field_72338_b, axisalignedbb.field_72339_c, axisalignedbb.field_72336_d, (double) ((float) i * 0.125F), axisalignedbb.field_72334_f);
    }

    @Override
    @SuppressWarnings("deprecation")
    public boolean func_149662_c(IBlockState state)
    {
        return state.func_177229_b(LAYERS) == 8;
    }

    @Override
    @SuppressWarnings("deprecation")
    public void func_189540_a(IBlockState state, World worldIn, BlockPos pos, Block blockIn, BlockPos fromPos)
    {
        if (!worldIn.field_72995_K)
        {
            // Try to drop the rock down
            IBlockState stateUnder = worldIn.func_180495_p(pos.func_177977_b());
            if (stateUnder.func_177230_c() instanceof BlockCharcoalPile)
            {
                int layersAt = state.func_177229_b(LAYERS);
                int layersUnder = stateUnder.func_177229_b(LAYERS);
                if (layersUnder < 8)
                {
                    if (layersUnder + layersAt <= 8)
                    {
                        worldIn.func_175656_a(pos.func_177977_b(), stateUnder.func_177226_a(LAYERS, layersAt + layersUnder));
                        worldIn.func_175698_g(pos);
                    }
                    else
                    {
                        worldIn.func_175656_a(pos.func_177977_b(), stateUnder.func_177226_a(LAYERS, 8));
                        worldIn.func_175656_a(pos, state.func_177226_a(LAYERS, layersAt + layersUnder - 8));
                    }
                }
                return;
            }

            if (!stateUnder.func_185915_l())
            {
                this.func_176226_b(worldIn, pos, state, 0);
                worldIn.func_175698_g(pos);
            }
        }
    }

    @Override
    public Item func_180660_a(IBlockState state, Random rand, int fortune)
    {
        return Items.field_151044_h;
    }

    @Override
    public int func_180651_a(IBlockState state)
    {
        return 1;
    }

    @Override
    public boolean func_180639_a(World world, BlockPos pos, IBlockState state, EntityPlayer player, EnumHand hand, EnumFacing side, float hitX, float hitY, float hitZ)
    {
        if (!world.field_72995_K)
        {

            ItemStack stack = player.func_184586_b(hand);

            // Special Interactions
            if (stack.func_77973_b() == Items.field_151044_h && stack.func_77960_j() == 1)
            {
                if (state.func_177229_b(LAYERS) < 8)
                {
                    world.func_175656_a(pos, state.func_177226_a(LAYERS, state.func_177229_b(LAYERS) + 1));
                    if (!player.func_184812_l_())
                    {
                        player.func_184611_a(hand, Helpers.consumeItem(stack, 1));
                    }
                    world.func_184133_a(null, pos, SoundEvents.field_187587_bZ, SoundCategory.BLOCKS, 1.0F, 0.3F);
                    return true;
                }
            }
            return false;
        }
        return true;
    }

    @Override
    protected BlockStateContainer func_180661_e()
    {
        return new BlockStateContainer(this, LAYERS);
    }

    @Override
    public boolean removedByPlayer(IBlockState state, World world, BlockPos pos, EntityPlayer player, boolean willHarvest)
    {
        this.func_176208_a(world, pos, state, player);

        if (player.func_184812_l_())
        {
            return super.removedByPlayer(state, world, pos, player, willHarvest);
        }

        if (!world.field_72995_K)
        {

            int layers = state.func_177229_b(LAYERS);
            if (layers == 1)
            {
                world.func_175698_g(pos);
            }
            else
            {
                world.func_175656_a(pos, state.func_177226_a(LAYERS, layers - 1));
            }
        }
        return true;
    }

    @Override
    public ItemStack getPickBlock(IBlockState state, RayTraceResult target, World world, BlockPos pos, EntityPlayer player)
    {
        return new ItemStack(Items.field_151044_h, 1, 1);
    }
}
