/*
 * Work under Copyright. Licensed under the EUPL.
 * See the project README.md and LICENSE.txt for more information.
 */

package net.dries007.tfc.objects.container;

import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.entity.player.InventoryPlayer;
import net.minecraft.inventory.Slot;
import net.minecraft.item.ItemStack;
import net.minecraft.util.EnumHand;
import net.minecraftforge.items.ItemStackHandler;

import net.dries007.tfc.api.registries.TFCRegistries;
import net.dries007.tfc.api.types.KnappingRecipe;
import net.dries007.tfc.objects.inventory.SlotKnappingOutput;
import net.dries007.tfc.util.Helpers;
import net.dries007.tfc.util.SimpleCraftMatrix;

public class ContainerKnapping extends ContainerItemStack
{
    private final SimpleCraftMatrix matrix;
    private final KnappingRecipe.Type type;
    private final ItemStack stackCopy;

    private boolean hasBeenModified;
    public boolean requiresReset;

    public ContainerKnapping(KnappingRecipe.Type type, InventoryPlayer playerInv, ItemStack stack)
    {
        super(playerInv, stack);
        this.itemIndex += 1;
        this.type = type;
        this.stackCopy = this.stack.func_77946_l();

        matrix = new SimpleCraftMatrix();
        hasBeenModified = false;
        requiresReset = false;
    }

    public void onUpdate(int slotIdx)
    {
        matrix.set(slotIdx, false);

        if (!hasBeenModified)
        {
            ItemStack stack = Helpers.consumeItem(this.stack, type.getAmountToConsume());
            if (isOffhand)
                player.func_184611_a(EnumHand.OFF_HAND, stack);
            else
                player.func_184611_a(EnumHand.MAIN_HAND, stack);
            hasBeenModified = true;
        }

        // check the pattern
        Slot slot = field_75151_b.get(0);
        if (slot != null)
        {
            KnappingRecipe recipe = getMatchingRecipe();
            if (recipe != null)
            {
                slot.func_75215_d(recipe.getOutput(this.stackCopy));
            }
            else
            {
                slot.func_75215_d(ItemStack.field_190927_a);
            }
        }
    }

    @Override
    public void func_75134_a(EntityPlayer player)
    {
        Slot slot = field_75151_b.get(0);
        ItemStack stack = slot.func_75211_c();
        if (!stack.func_190926_b())
        {
            player.func_191521_c(stack);
        }
        super.func_75134_a(player);
    }

    @Override
    protected void addContainerSlots()
    {
        func_75146_a(new SlotKnappingOutput(new ItemStackHandler(1), 0, 128, 44, this::resetMatrix));
    }

    private void resetMatrix()
    {
        matrix.setAll(false);
        requiresReset = true;
    }

    private KnappingRecipe getMatchingRecipe()
    {
        return TFCRegistries.KNAPPING.getValuesCollection()
            .stream()
            .filter(x -> x.getType() == type && matrix.matches(x.getMatrix()))
            .findFirst()
            .orElse(null);
    }

    @Override
    protected void addPlayerInventorySlots(InventoryPlayer playerInv)
    {
        // Add Player Inventory Slots (lower down)
        for (int i = 0; i < 3; i++)
        {
            for (int j = 0; j < 9; j++)
            {
                func_75146_a(new Slot(playerInv, j + i * 9 + 9, 8 + j * 18, 84 + i * 18 + 18));
            }
        }

        for (int k = 0; k < 9; k++)
        {
            func_75146_a(new Slot(playerInv, k, 8 + k * 18, 142 + 18));
        }
    }
}
