/*
 * Work under Copyright. Licensed under the EUPL.
 * See the project README.md and LICENSE.txt for more information.
 */

package net.dries007.tfc.objects.items;

import java.util.ArrayList;
import java.util.List;
import javax.annotation.Nullable;
import javax.annotation.ParametersAreNonnullByDefault;

import net.minecraft.block.material.Material;
import net.minecraft.block.state.IBlockState;
import net.minecraft.entity.Entity;
import net.minecraft.entity.EntityLivingBase;
import net.minecraft.entity.item.EntityItem;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.init.Blocks;
import net.minecraft.init.Items;
import net.minecraft.item.EnumAction;
import net.minecraft.item.Item;
import net.minecraft.item.ItemFlintAndSteel;
import net.minecraft.item.ItemStack;
import net.minecraft.util.EnumActionResult;
import net.minecraft.util.EnumFacing;
import net.minecraft.util.EnumHand;
import net.minecraft.util.EnumParticleTypes;
import net.minecraft.util.math.AxisAlignedBB;
import net.minecraft.util.math.BlockPos;
import net.minecraft.util.math.RayTraceResult;
import net.minecraft.world.World;

import mcp.MethodsReturnNonnullByDefault;
import net.dries007.tfc.api.capability.size.Size;
import net.dries007.tfc.api.capability.size.Weight;
import net.dries007.tfc.objects.blocks.BlocksTFC;
import net.dries007.tfc.objects.te.TEFirePit;
import net.dries007.tfc.objects.te.TELogPile;
import net.dries007.tfc.objects.te.TEPitKiln;
import net.dries007.tfc.util.Helpers;
import net.dries007.tfc.util.OreDictionaryHelper;

import static net.dries007.tfc.objects.blocks.BlockFirePit.LIT;
import static net.dries007.tfc.objects.blocks.wood.BlockLogPile.ONFIRE;

@MethodsReturnNonnullByDefault
@ParametersAreNonnullByDefault
public class ItemFireStarter extends ItemTFC
{
    public static boolean canIgnite(ItemStack stack)
    {
        if (stack.func_190926_b()) return false;
        Item item = stack.func_77973_b();

        //noinspection ConstantConditions
        return item == ItemsTFC.FIRESTARTER || item == Items.field_151033_d || item == Items.field_151059_bz || item instanceof ItemFlintAndSteel;
    }

    public ItemFireStarter()
    {
        func_77656_e(8);
        func_77625_d(1);
        setNoRepair();
    }

    @Override
    public EnumActionResult func_180614_a(EntityPlayer player, World worldIn, BlockPos pos, EnumHand hand, EnumFacing facing, float hitX, float hitY, float hitZ)
    {
        if (worldIn.field_72995_K) return EnumActionResult.PASS;
        if (hand != EnumHand.MAIN_HAND) return EnumActionResult.PASS;
        if (canStartFire(worldIn, player) == null) return EnumActionResult.FAIL;
        player.func_184598_c(hand);
        return EnumActionResult.SUCCESS;
    }

    @Override
    public EnumAction func_77661_b(ItemStack stack)
    {
        return EnumAction.BOW;
    }

    @Override
    public int func_77626_a(ItemStack stack)
    {
        return 72;
    }

    @Override
    @SuppressWarnings("ConstantConditions")
    public void onUsingTick(ItemStack stack, EntityLivingBase entityLivingBase, int countLeft)
    {
        if (!(entityLivingBase instanceof EntityPlayer)) return;
        final EntityPlayer player = ((EntityPlayer) entityLivingBase);
        final RayTraceResult result = canStartFire(player.field_70170_p, player);
        if (result == null)
        {
            player.func_184602_cy();
            return;
        }
        final int total = func_77626_a(stack);
        final int count = total - countLeft;
        final BlockPos pos = result.func_178782_a().func_177982_a(0, 1, 0);
        final World world = player.field_70170_p;
        final float chance = field_77697_d.nextFloat(); // todo: raining etc?

        if (world.field_72995_K) // Client
        {
            if (chance > 0.7 && field_77697_d.nextFloat() < count / (double) total)
                world.func_175688_a(EnumParticleTypes.SMOKE_NORMAL, result.field_72307_f.field_72450_a, result.field_72307_f.field_72448_b, result.field_72307_f.field_72449_c, 0.0F, 0.1F, 0.0F);
            if (chance > 0.7 && countLeft < 10 && field_77697_d.nextFloat() < count / (double) total)
                world.func_175688_a(EnumParticleTypes.FLAME, result.field_72307_f.field_72450_a, result.field_72307_f.field_72448_b, result.field_72307_f.field_72449_c, 0.0F, 0.0F, 0.0F);
        }
        else if (countLeft == 1) // Server, and last tick of use
        {
            final IBlockState state = world.func_180495_p(pos.func_177977_b());
            if (state.func_177230_c() == BlocksTFC.LOG_PILE)
            {
                // Log pile
                if (Math.random() < chance)
                {
                    world.func_175656_a(pos.func_177977_b(), state.func_177226_a(ONFIRE, true));
                    TELogPile te = Helpers.getTE(world, pos.func_177977_b(), TELogPile.class);
                    if (te != null) te.light();
                    if (!Blocks.field_150480_ab.func_176196_c(world, pos))
                        world.func_175656_a(pos, Blocks.field_150480_ab.func_176223_P());
                }
            }
            else if (state.func_177230_c() == BlocksTFC.PIT_KILN)
            {
                // Pit Kiln
                if (Math.random() < chance)
                {
                    TEPitKiln te = Helpers.getTE(world, pos.func_177977_b(), TEPitKiln.class);
                    if (te != null) te.tryLight();
                }
            }
            else
            {
                // Try fire pit
                stack.func_77972_a(1, player);

                final List<EntityItem> items = world.func_72872_a(EntityItem.class, new AxisAlignedBB(pos, pos.func_177982_a(1, 2, 1)));
                final List<EntityItem> stuffToUse = new ArrayList<>();

                int sticks = 3, kindling = 1;
                EntityItem log = null;

                for (EntityItem entity : items)
                {
                    if (OreDictionaryHelper.doesStackMatchOre(entity.func_92059_d(), "stickWood"))
                    {
                        sticks -= entity.func_92059_d().func_190916_E();
                        stuffToUse.add(entity);
                    }
                    else if (OreDictionaryHelper.doesStackMatchOre(entity.func_92059_d(), "kindling"))
                    {
                        kindling -= entity.func_92059_d().func_190916_E();
                        stuffToUse.add(entity);
                    }
                    else if (log == null && OreDictionaryHelper.doesStackMatchOre(entity.func_92059_d(), "logWood"))
                    {
                        log = entity;
                    }
                }

                if (sticks <= 0 && kindling <= 0 && log != null)
                {
                    final float kindlingModifier = Math.min(-0.1f * (float) kindling, 0.5f);
                    if (field_77697_d.nextFloat() < chance + kindlingModifier)
                    {
                        //noinspection ConstantConditions
                        world.func_175656_a(pos, BlocksTFC.FIREPIT.func_176223_P().func_177226_a(LIT, true));
                        TEFirePit te = Helpers.getTE(world, pos, TEFirePit.class);
                        if (te != null)
                        {
                            te.onCreate(log.func_92059_d());
                        }
                        stuffToUse.forEach(Entity::func_70106_y);
                        log.func_92059_d().func_190918_g(1);
                        if (log.func_92059_d().func_190916_E() == 0)
                            log.func_70106_y();
                    }
                }
                else
                {
                    // Else, start a fire
                    // todo: start a fire
                }
            }
        }
    }

    @Override
    public Size getSize(ItemStack stack)
    {
        return Size.SMALL;
    }

    @Override
    public Weight getWeight(ItemStack stack)
    {
        return Weight.LIGHT;
    }

    @Nullable
    private RayTraceResult canStartFire(World world, EntityPlayer player)
    {
        RayTraceResult result = func_77621_a(world, player, true);
        //noinspection ConstantConditions
        if (result == null) return null;
        if (result.field_72313_a != RayTraceResult.Type.BLOCK) return null;
        BlockPos pos = result.func_178782_a();
        final IBlockState current = player.field_70170_p.func_180495_p(pos);
        if (result.field_178784_b != EnumFacing.UP) return null;
        if (!current.isSideSolid(world, pos, EnumFacing.UP)) return null;
        if (current.func_185904_a() == Material.field_151586_h) return null;
        pos = pos.func_177982_a(0, 1, 0);
        if (!world.func_175623_d(pos)) return null;
        return result;
    }
}
