/*
 * Work under Copyright. Licensed under the EUPL.
 * See the project README.md and LICENSE.txt for more information.
 */

package net.dries007.tfc.objects.te;

import javax.annotation.ParametersAreNonnullByDefault;

import net.minecraft.block.Block;
import net.minecraft.block.state.IBlockState;
import net.minecraft.init.Blocks;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.util.EnumFacing;
import net.minecraft.util.ITickable;
import net.minecraft.util.math.BlockPos;
import net.minecraft.world.World;

import mcp.MethodsReturnNonnullByDefault;
import net.dries007.tfc.objects.blocks.BlockCharcoalPile;
import net.dries007.tfc.objects.blocks.BlocksTFC;
import net.dries007.tfc.objects.blocks.wood.BlockLogPile;
import net.dries007.tfc.util.Helpers;
import net.dries007.tfc.util.OreDictionaryHelper;

import static net.dries007.tfc.objects.blocks.BlockCharcoalPile.LAYERS;
import static net.dries007.tfc.objects.blocks.wood.BlockLogPile.ONFIRE;

@ParametersAreNonnullByDefault
@MethodsReturnNonnullByDefault
public class TELogPile extends TESidedInventory implements ITickable
{
    private static final int NUM_SLOTS = 4;

    public static boolean isStackValid(ItemStack stack)
    {
        return (stack.func_190926_b() || OreDictionaryHelper.doesStackMatchOre(stack, "logWood"));
    }

    private final int maxBurnTicks = 8000; // 8 In-game Hours
    public boolean burning;
    private int burnTicks;

    public TELogPile()
    {
        super(NUM_SLOTS);

        burnTicks = 0;
        burning = false;
    }

    @Override
    public void func_73660_a()
    {
        if (field_145850_b.field_72995_K) { return; }

        if (burning)
        {
            if (burnTicks < maxBurnTicks)
            {
                burnTicks++;
            }
            else
            {
                // Attempt to turn this log pile into charcoal
                createCharcoal();
            }
        }
        else
        {
            if (field_145850_b.func_180495_p(field_174879_c.func_177984_a()).func_177230_c() == Blocks.field_150480_ab)
            {
                burning = true;
            }
        }
    }

    @Override
    public void setAndUpdateSlots(int slot)
    {
        if (!field_145850_b.field_72995_K)
        {
            for (int i = 0; i < 4; i++)
            {
                if (!inventory.getStackInSlot(i).func_190926_b())
                {
                    super.setAndUpdateSlots(slot);
                    return;
                }
            }
            field_145850_b.func_175698_g(field_174879_c);
        }
        super.setAndUpdateSlots(slot);
    }

    @Override
    public int getSlotLimit(int slot)
    {
        return 4;
    }

    @Override
    public boolean isItemValid(int slot, ItemStack stack)
    {
        return isStackValid(stack);
    }

    @Override
    public void func_145839_a(NBTTagCompound c)
    {
        burnTicks = c.func_74762_e("burn_ticks");
        burning = c.func_74767_n("burning");
        super.func_145839_a(c);
    }

    @Override
    public NBTTagCompound func_189515_b(NBTTagCompound c)
    {
        c.func_74768_a("burn_ticks", burnTicks);
        c.func_74757_a("burning", burning);
        return super.func_189515_b(c);
    }

    public boolean insertLog(ItemStack stack)
    {
        stack.func_190920_e(1);
        for (int i = 0; i < inventory.getSlots(); i++)
        {
            if (inventory.insertItem(i, stack, false).func_190926_b())
            {
                return true;
            }
        }
        return false;
    }

    public ItemStack getLog()
    {
        for (int i = 0; i < inventory.getSlots(); i++)
        {
            if (!inventory.getStackInSlot(i).func_190926_b())
            {
                return inventory.getStackInSlot(i);
            }
        }
        return ItemStack.field_190927_a;
    }

    public void light()
    {
        this.burning = true;
        tryLightNearby();
    }

    public int countLogs()
    {
        int logs = 0;
        for (int i = 0; i < inventory.getSlots(); i++)
        {
            logs += inventory.getStackInSlot(i).func_190916_E();
        }
        return logs;
    }

    @Override
    public boolean shouldRefresh(World world, BlockPos pos, IBlockState oldState, IBlockState newState)
    {
        return (oldState.func_177230_c() != newState.func_177230_c());
    }

    private void tryLightNearby()
    {
        for (EnumFacing side : EnumFacing.values())
        {
            IBlockState state = field_145850_b.func_180495_p(field_174879_c.func_177972_a(side));
            if (state.func_177230_c() instanceof BlockLogPile)
            {
                if (state.func_177229_b(ONFIRE)) continue;
                field_145850_b.func_175656_a(field_174879_c.func_177972_a(side), state.func_177226_a(ONFIRE, true));
                TELogPile tile = Helpers.getTE(field_145850_b, field_174879_c.func_177972_a(side), TELogPile.class);
                if (tile != null)
                {
                    tile.light();
                }
            }
        }
    }

    // This function does some magic **** to not create floating charcoal. Don't touch unless broken
    private void createCharcoal()
    {
        int j = 0;
        Block block;
        do
        {
            j++;
            block = field_145850_b.func_180495_p(field_174879_c.func_177979_c(j)).func_177230_c();
            // This is here so that the charcoal pile will collapse Bottom > Top
            // Because the pile scans Top > Bottom this is nessecary to avoid floating blocks
            if (block instanceof BlockLogPile) { return; }
        } while (block == Blocks.field_150350_a || block instanceof BlockCharcoalPile);

        double logs = (double) countLogs();
        double log2 = 0.008d * logs * (logs + 42.5d) - 0.75d + 1.5d * Math.random();
        int charcoal = (int) Math.min(8, Math.max(0, Math.round(log2)));
        if (charcoal == 0)
        {
            field_145850_b.func_175656_a(field_174879_c, Blocks.field_150350_a.func_176223_P());
            return;
        }
        if (j == 1)
        {
            // This log pile is at the bottom of the charcoal pit
            //noinspection ConstantConditions
            field_145850_b.func_175656_a(field_174879_c, BlocksTFC.CHARCOAL_PILE.func_176223_P().func_177226_a(LAYERS, charcoal));
            return;
        }
        for (int k = j - 1; k >= 0; k--)
        {
            // Climb back up from the bottom
            IBlockState state = field_145850_b.func_180495_p(field_174879_c.func_177979_c(k));
            if (state.func_177230_c() == Blocks.field_150350_a)
            {
                // If it hits air, place the remaining pile in that block
                //noinspection ConstantConditions
                field_145850_b.func_175656_a(field_174879_c.func_177979_c(k), BlocksTFC.CHARCOAL_PILE.func_176223_P().func_177226_a(LAYERS, charcoal));
                field_145850_b.func_175656_a(field_174879_c, Blocks.field_150350_a.func_176223_P());
                return;
            }

            if (state.func_177230_c() instanceof BlockCharcoalPile)
            {
                // Place what it can in the existing charcoal pit, then continue climbing
                charcoal += state.func_177229_b(LAYERS);
                int toCreate = charcoal > 8 ? 8 : charcoal;
                //noinspection ConstantConditions
                field_145850_b.func_175656_a(field_174879_c.func_177979_c(k), BlocksTFC.CHARCOAL_PILE.func_176223_P().func_177226_a(LAYERS, toCreate));
                charcoal -= toCreate;
            }

            if (charcoal <= 0)
            {
                field_145850_b.func_175656_a(field_174879_c, Blocks.field_150350_a.func_176223_P());
                return;
            }
        }
        // If you exit the loop, its arrived back at the original position OR needs to rest the original position, and needs to replace that block
        //noinspection ConstantConditions
        field_145850_b.func_175656_a(field_174879_c, BlocksTFC.CHARCOAL_PILE.func_176223_P().func_177226_a(LAYERS, charcoal));
    }
}
